import datetime as dt
import os
from typing import Any

from notifier.frigate.config import FrigateConfig, EventType
from notifier.frigate.mqtt import MQTTSubscriber
from notifier.notify.ntfy import NtfyNotification, NtfyNotificationSender


def on_event(payload: dict[str, Any]) -> None:
    event_type = EventType(payload["type"])
    camera = payload["after"]["camera"]
    object_label = payload["after"]["label"]
    new_zones = set(payload["after"]["entered_zones"]) - set(payload["before"]["entered_zones"])
    for new_zone in new_zones:
        if frigate_config.is_alert(event_type, camera, new_zone, object_label):
            event_id = payload["after"]["id"]
            score = payload["after"]["top_score"]

            notification = NtfyNotification(event_id, camera, object_label, score)
            notification_sender.send(notification)


if __name__ == "__main__":
    frigate_config = FrigateConfig()
    notification_sender = NtfyNotificationSender(quiet_period=dt.timedelta(seconds=3 * 60))
    mqtt_subscriber = MQTTSubscriber(on_event, username=os.getenv("MQTT_USERNAME"), password=os.getenv("MQTT_PASSWORD"))
    mqtt_subscriber.subscribe()

