import json
import logging
from typing import Any, Callable

from paho.mqtt.client import Client as MQTTClient, MQTTMessage
from paho.mqtt.reasoncodes import ReasonCode

logger = logging.getLogger(__name__)


class MQTTSubscriber:
    def __init__(self, callback: Callable[[dict[str, Any]], None], host: str = "mqtt", port: int = 1883, username: str | None = None, password: str | None = None) -> None:
        self._callback = callback
        self._host = host
        self._port = port
        self._client = MQTTClient()
        if username and password:
            self._client.username_pw_set(username, password=password)

    def subscribe(self) -> None:
        self._client.connect(host=self._host, port=self._port)
        self._client.loop_forever()

    def _on_connect(self, client: MQTTClient, userdata: Any, flags: dict[str, Any], rc: ReasonCode) -> None:
        logger.info("Connected with return code {rc}")
        client.subscribe("frigate/events")

    def _on_message(self, client: MQTTClient, userdata: Any, message: MQTTMessage) -> None:
        payload = json.loads(message.payload.decode())
        self._callback(payload)
